from .types import SymbolInfo, TickTrade, SkippedDay, Kline, Funding, PremiumIndexKline, SymbolVolatility
from typing import overload, Literal


@overload
def format_trades(raw_trades: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_trades(raw_trades: list, raw: Literal[False]) -> list[TickTrade]: ...
@overload
def format_trades(raw_trades: list, raw: bool) -> list[dict] | list[TickTrade]: ...


def format_trades(raw_trades, raw=False) -> list[dict] | list[TickTrade]:
    if not raw_trades:
        return []

    trades = [
        {
            "time": t[0],
            "symbol": t[1],
            "price": t[2],
            "quantity": t[3],
        }
        for t in raw_trades
    ]

    if raw:
        return trades

    return [TickTrade.model_validate(trade) for trade in trades]


@overload
def format_symbols(raw_symbols: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_symbols(raw_symbols: list, raw: Literal[False]) -> list[SymbolInfo]: ...
@overload
def format_symbols(raw_symbols: list, raw: bool) -> list[dict] | list[SymbolInfo]: ...


def format_symbols(raw_symbols, raw=False) -> list[dict] | list[SymbolInfo]:
    if not raw_symbols:
        return []

    symbols = [
        {
            "symbol": t[0],
            "first_data_date": t[1],
            "last_loaded_data_date": t[2],
            "is_spot_finished": t[3],
            "first_futures_data_date": t[4],
            "last_loaded_futures_data_date": t[5],
            "is_futures_finished": t[6],
        }
        for t in raw_symbols
    ]

    if raw:
        return symbols

    return [SymbolInfo.model_validate(symbol) for symbol in symbols]


@overload
def format_funding(raw_funding: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_funding(raw_funding: list, raw: Literal[False]) -> list[Funding]: ...
@overload
def format_funding(raw_funding: list, raw: bool) -> list[dict] | list[Funding]: ...


def format_funding(raw_funding, raw=False) -> list[dict] | list[Funding]:
    if not raw_funding:
        return []

    fundings = [{"symbol": t[0], "time": t[1], "funding": t[2], "mark_price": t[3]} for t in raw_funding]

    if raw:
        return fundings

    return [Funding.model_validate(funding) for funding in fundings]


@overload
def format_premium_index(raw_premium_index: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_premium_index(raw_premium_index: list, raw: Literal[False]) -> list[PremiumIndexKline]: ...
@overload
def format_premium_index(raw_premium_index: list, raw: bool) -> list[dict] | list[PremiumIndexKline]: ...


def format_premium_index(raw_premium_index, raw=False) -> list[dict] | list[PremiumIndexKline]:
    if not raw_premium_index:
        return []

    premium_indexes = [
        {
            "symbol": t[0],
            "time": t[1],
            "open": t[2],
            "high": t[3],
            "low": t[4],
            "close": t[5],
        }
        for t in raw_premium_index
    ]

    if raw:
        return premium_indexes

    return [PremiumIndexKline(premium_index) for premium_index in premium_indexes]


@overload
def format_skipped_days(raw_skipped_days: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_skipped_days(raw_skipped_days: list, raw: Literal[False]) -> list[SkippedDay]: ...
@overload
def format_skipped_days(raw_skipped_days: list, raw: bool) -> list[dict] | list[SkippedDay]: ...


def format_skipped_days(raw_skipped_days, raw=False) -> list[dict] | list[SkippedDay]:
    if not raw_skipped_days:
        return []

    skipped_days = [
        {
            "symbol": t[0],
            "type": t[1],
            "date": t[2],
        }
        for t in raw_skipped_days
    ]

    if raw:
        return skipped_days

    return [SkippedDay(skipped_day) for skipped_day in skipped_days]


@overload
def format_ohlc_prices(raw_ohlc: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_ohlc_prices(raw_ohlc: list, raw: Literal[False]) -> list[Kline]: ...
@overload
def format_ohlc_prices(raw_ohlc: list, raw: bool) -> list[dict] | list[Kline]: ...


def format_ohlc_prices(raw_ohlc, raw=False) -> list[dict] | list[Kline]:
    if not raw_ohlc:
        return []

    ohlc = [
        {
            "symbol": t[0],
            "date": t[1],
            "open": t[2],
            "high": t[3],
            "low": t[4],
            "close": t[5],
            "volume": t[6],
        }
        for t in raw_ohlc
    ]

    if raw:
        return ohlc

    return [Kline.model_validate(o) for o in ohlc]


@overload
def format_symbol_volatility(raw_volatility: list, raw: Literal[True]) -> list[dict]: ...
@overload
def format_symbol_volatility(raw_volatility: list, raw: Literal[False]) -> list[SymbolVolatility]: ...
@overload
def format_symbol_volatility(raw_volatility: list, raw: bool) -> list[dict] | list[SymbolVolatility]: ...


def format_symbol_volatility(raw_volatility, raw=False) -> list[dict] | list[SymbolVolatility]:
    if not raw_volatility:
        return []

    volatility = [
        {
            "symbol": t[0],
            "date": t[1],
            "long_trades": t[2],
            "short_trades": t[3],
            "last_level_price": t[4],
        }
        for t in raw_volatility
    ]

    if raw:
        return volatility

    return [SymbolVolatility.model_validate(vol) for vol in volatility]
